/******************************************************************************
 * 	Copyright (c) 1997-2004 Mark Hummel DBA Raquette Software.
 *		All rights reserved
 *
 *    This file contains source code written by Raquette Software, 
 *    68 Stewart Street, Franklin MA 02038. It may not be used without 
 *    express written permission. The expression of the information 
 *    contained herein is protected under federal copyright laws and 
 *    all copying without permission is prohibited and may be subject 
 *    to criminal penalties. The Author assumes no responsibility for 
 *    errors, omissions, or damages caused by the use of these programs 
 *    or from use of the information contained herein.
 *
 *-----------------------------------------------------------------------------
 */
/******************************************************************************
 *
 *
 *	   compile.cpp
 *		- library of compile related routines
 *
 ******************************************************************************
 */
						

#include <vector>
#include <fstream>
#include "compile.h"
#include "cnode.h"
#include "cmodule.h"
#include "cinstance.h"


/************************************************
	CreateModuleList
	- mapping of name to module definition
 ************************************************/

static CNode* moduleComments;

static void CreateModuleList( 
    CNode*                n, 
    set<CModule*>&        allModules,
    set<CModule*>&        leafModules,
    map<CModule*,CNode*>& module2Comments
)
{
    if( n == NULL ) {
	return;
    }
    switch( n->GetOp() ) {
    case eLIST:
	CreateModuleList( n->Arg<CNode*>(0), allModules, leafModules, module2Comments );
	CreateModuleList( n->Arg<CNode*>(1), allModules, leafModules, module2Comments );
	break;
    case eMODULE_DEF:
	{
	    CModule* m = n->Arg<CModule*>(0);
	    module2Comments[m] = moduleComments;
	    moduleComments = NULL;
	    allModules.insert(m);
	    vector<CInstance*>& inst = *m->GetInstanceList();
	    vector<CInstance*>::iterator p;
	    for( p = inst.begin(); p != inst.end(); ++p ) {
		CModule* def = (*p)->GetDefinition();
		leafModules.insert(def);
	    }
	}
	break;
    case eVRQ:
    case ePRAGMA:
    case eATTRIBUTE:
	break;
    case eCOMMENT:
	moduleComments = cLINK( moduleComments, n );
	break;
    default:
	MASSERT( FALSE );
    }       
}

/************************************************
	AnalyseModules
	- analyse module hierarchy
 ************************************************/

void AnalyseModules( 
    CNode* codeList, 
    set<CModule*>& allModules,
    set<CModule*>& topLevelModules,
    set<CModule*>& leafModules,
    set<CModule*>& undefinedModules,
    map<CModule*,CNode*>& module2Comments
)
{
    moduleComments = NULL;
    CreateModuleList( codeList, allModules, leafModules, module2Comments );
    
    set_difference( allModules.begin(), allModules.end(),
		    leafModules.begin(), leafModules.end(),
		    inserter(topLevelModules, topLevelModules.end()) );

    set_difference( leafModules.begin(), leafModules.end(),
			allModules.begin(), allModules.end(),
			inserter(undefinedModules, undefinedModules.end()) );
	
}

